<div class="panel panel-default">
  <div class="panel-heading">
    <h3 class="panel-title">{{translateTitle(currentNode)}}</h3>
  </div>
  <table class="table table-striped">
    <thead>
      <tr>
        <th trspan="name"></th>
        <th trspan="authentication"></th>
        <th trspan="userDB"></th>
        <th trspan="passwordDB"></th>
        <th trspan="url"></th>
      </tr>
    </thead>
    <tbody>
      <tr ng-repeat="c in currentNode.nodes">
        <td>
          <input class="form-control" ng-model="c.title">
        </td>
        <td ng-repeat="i in [0,1,2]">
          <select class="form-control" ng-model="c.data[i]" ng-change="execFilters(_findScopeByKey('authParams'))">
            <option ng-repeat="item in currentNode.select[i]" ng-selected="item.k==c.data[i]" value="{{item.k}}">{{item.v}}</option>
          </select>
        </td>
        <td>
          <input class="form-control" ng-model="c.data[3]">
        </td>
      </tr>
    </tbody>
  </table>
</div>
<script type="text/menu">
[{
  "action": "newAuthChoice",
  "title": "newChain",
  "icon": "plus-sign"
}]
</script>
