<div class="panel panel-default">
  <div class="panel-heading">
    <h3 class="panel-title">{{translateTitle(currentNode)}}</h3>
  </div>
  <div class="panel-body">
    <div class="input-group-solid">
      <label class="radio-inline">
        <input id="bopeOn" type="radio" value="1" name="bope" ng-model="currentNode.data"/>
        <span trspan="on"></span>
      </label>
      <label class="radio-inline">
        <input id="bopeOff" type="radio" value="0" name="bope" ng-model="currentNode.data" ng-checked="currentNode.data!==1&&currentNode.data!=='1'"/>
        <span trspan="off"></span>
      </label>
    </div>
  </div>
</div>
<script type="text/menu">
[{
  "title": "cancel",
  "icon": "remove"
}]
</script>
