<div class="panel panel-default llcontainer">
  <div class="panel-heading">
    <h3 class="panel-title">{{translateTitle(currentNode)}}</h3>
  </div>
  <div class="panel-body">
    <div class="input-group-solid">
      <label class="radio-inline">
        <input id="bopeOn" type="radio" value="1" name="bope" ng-model="currentNode.data">
        <span trspan="on"></span>
      </label>
      <label class="radio-inline">
        <input id="bopeOff" type="radio" value="0" name="bope" ng-model="currentNode.data">
        <span trspan="off"></span>
      </label>
      <label class="radio-inline">
        <input id="bopeExpr" type="radio" value="-1" name="bope" ng-click="currentNode.data=''" ng-checked="currentNode.data!==0&&currentNode.data!=='0'&&currentNode.data!==1&&currentNode.data!=='1'">
        <span trspan="specialRule"></span>
      </label>
    </div>
  </div>
  <div class="panel-body input-group maxw" ng-if="currentNode.data!==0&&currentNode.data!=='0'&&currentNode.data!==1&&currentNode.data!=='1'">
    <textarea id="bopeValue" class="form-control" rows="2" ng-model="currentNode.data"/>
  </div>
</div>
<script type="text/menu">
[{
  "title": "cancel",
  "icon": "remove"
}]
</script>
